{*******************************************************************************
  Copyright (C) Michael W. Vogel m-w-vogel@gmx.de

  The contents of this file are subject to the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 3.0 of the License,
  or (at your option) any later version.
  You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
  specific language governing rights and limitations under the License.

  Created 02.09.2015
*******************************************************************************}

unit ArchiveParser;

{$mode objfpc}{$H+}

interface

function IsDirectory(const aPackFileName: String): Boolean;     //Ist PackFileName ein Verzeichnis
function IsFile(const aPackFileName: String): Boolean;          //Ist PackFileName eine Datei
function DiskFileName(const aPackFileName: String): String;     //Extrahiert DiskFileName aus String
function ArchiveFileName(const aPackFileName: String): String;  //Extrahiert ArchivFileName aus String
function PackFileName(aIsDirectory: Boolean;                    //Erstellt PackFileNamen - String
  const aDiskFileName, aArchiveFileName: String): String;

implementation

function IsDirectory(const aPackFileName: String): Boolean;
begin
  if (Length(aPackFileName) > 1) then
    Result := aPackFileName[1] = 'D'
  else
    Result := False;
end;

function IsFile(const aPackFileName: String): Boolean;
begin
  if (Length(aPackFileName) > 1) then
    Result := aPackFileName[1] = 'F'
  else
    Result := False;
end;

function DiskFileName(const aPackFileName: String
  ): String;
var
  Cnt, aPos, Pos1, Pos2: Integer;
begin
  Pos1 := -1;
  Pos2 := -1;
  Cnt := 0;
  aPos := 1;
  while (aPos <= Length(aPackFileName)) and (Cnt < 1) do begin
    if aPackFileName[aPos] = '[' then Inc(Cnt);
    if aPackFileName[aPos] = ']' then Dec(Cnt);
    if Cnt = 1 then Pos1 := aPos + 1;
    Inc(aPos);
  end;
  while (aPos <= Length(aPackFileName)) and (Cnt > 0) do begin
    if aPackFileName[aPos] = '[' then Inc(Cnt);
    if aPackFileName[aPos] = ']' then Dec(Cnt);
    if Cnt = 0 then Pos2 := aPos;
    Inc(aPos);
  end;
  if (Pos1 > 0) and (Pos2 > 0) then
    Result := Copy(aPackFileName, Pos1, Pos2 - Pos1)
  else
    Result := '';
end;

function ArchiveFileName(const aPackFileName: String
  ): String;
var
  Cnt, aPos, Pos1, Pos2: Integer;
begin
  Pos1 := -1;
  Pos2 := -1;
  Cnt := 0;
  aPos := 1;
  while (aPos <= Length(aPackFileName)) and (Cnt < 1) do begin
    if aPackFileName[aPos] = '[' then Inc(Cnt);
    if aPackFileName[aPos] = ']' then Dec(Cnt);
    Inc(aPos);
  end;
  while (aPos <= Length(aPackFileName)) and (Cnt > 0) do begin
    if aPackFileName[aPos] = '[' then Inc(Cnt);
    if aPackFileName[aPos] = ']' then Dec(Cnt);
    Inc(aPos);
  end;
  while (aPos <= Length(aPackFileName)) and (Cnt < 1) do begin
    if aPackFileName[aPos] = '[' then Inc(Cnt);
    if aPackFileName[aPos] = ']' then Dec(Cnt);
    if Cnt = 1 then Pos1 := aPos + 1;
    Inc(aPos);
  end;
  while (aPos <= Length(aPackFileName)) and (Cnt > 0) do begin
    if aPackFileName[aPos] = '[' then Inc(Cnt);
    if aPackFileName[aPos] = ']' then Dec(Cnt);
    if Cnt = 0 then Pos2 := aPos;
    Inc(aPos);
  end;
  if (Pos1 > 0) and (Pos2 > 0) then
    Result := Copy(aPackFileName, Pos1, Pos2 - Pos1)
  else
    Result := '';
end;

function PackFileName(aIsDirectory: Boolean;
  const aDiskFileName, aArchiveFileName: String): String;
begin
  if aIsDirectory then
    Result := 'D[' + aDiskFileName + '][' + aArchiveFileName + ']'
  else
    Result := 'F[' + aDiskFileName + '][' + aArchiveFileName + ']';
end;

end.

