{*******************************************************************************
  Copyright (C) Michael W. Vogel m-w-vogel@gmx.de

  The contents of this file are subject to the GNU Lesser General Public License
  as published by the Free Software Foundation; either version 3.0 of the License,
  or (at your option) any later version.
  You may obtain a copy of the LGPL at http://www.gnu.org/copyleft/.

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
  specific language governing rights and limitations under the License.

  Created 02.09.2015
*******************************************************************************}

unit PointerLists;

{$mode objfpc}{$H+}

interface

type
  TPointerListItem = Record
    Addr: Pointer;
    Next: Pointer;
  end;
  PPointerListItem = ^TPointerListItem;

  { TPointerList }

  TPointerList = class
    FFirst: Pointer;
    FLast: Pointer;
    FCurrent: Pointer;
  private
    function GetFirst: Pointer;
    function GetNext: Pointer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(p: Pointer);
    property First: Pointer read GetFirst;
    property Next: Pointer read GetNext;
  end;

implementation

{ TPointerList }

function TPointerList.GetFirst: Pointer;
begin
  FCurrent := FFirst;
  if Assigned(FCurrent) then
    Result := PPointerListItem(FCurrent)^.Addr
  else
    Result := nil;
end;

function TPointerList.GetNext: Pointer;
begin
  if not Assigned(FCurrent) then begin
    Result := nil;
    Exit;
  end;
  FCurrent := PPointerListItem(FCurrent)^.Next;
  if Assigned(FCurrent) then
    Result := PPointerListItem(FCurrent)^.Addr
  else
    Result := nil;
end;

constructor TPointerList.Create;
begin
  FCurrent := nil;
  FFirst   := nil;
  FLast    := nil;
end;

destructor TPointerList.Destroy;
var
  PLI, Dummy: PPointerListItem;
begin
  PLI := FFirst;
  while Assigned(PLI) do begin
    Dummy := PLI^.Next;
    FreeMem(PLI);
    PLI := Dummy;
  end;
end;

procedure TPointerList.Add(p: Pointer);
var
  PLI: PPointerListItem;
begin
  PLI := GetMem(SizeOf(TPointerListItem));
  PLI^.Addr := p;
  PLI^.Next := nil;
  if Assigned(FLast) then
    PPointerListItem(FLast)^.Next := PLI;
  FLast := PLI;
  if not Assigned(FFirst) then
    FFirst := PLI;
end;

end.

