unit uPointerList;

{$mode objfpc}{$H+}

interface

type
  TPointerListItem = Record
    Addr: Pointer;
    Next: Pointer;
  end;
  PPointerListItem = ^TPointerListItem;

  { TPointerList }

  TPointerList = class
    FFirst: Pointer;
    FLast: Pointer;
    FCurrent: Pointer;
  private
    function GetFirst: Pointer;
    function GetNext: Pointer;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Add(p: Pointer);
    property First: Pointer read GetFirst;
    property Next: Pointer read GetNext;
  end;

implementation

{ TPointerList }

function TPointerList.GetFirst: Pointer;
begin
  FCurrent := FFirst;
  if Assigned(FCurrent) then
    Result := PPointerListItem(FCurrent)^.Addr
  else
    Result := nil;
end;

function TPointerList.GetNext: Pointer;
begin
  if not Assigned(FCurrent) then begin
    Result := nil;
    Exit;
  end;
  FCurrent := PPointerListItem(FCurrent)^.Next;
  if Assigned(FCurrent) then
    Result := PPointerListItem(FCurrent)^.Addr
  else
    Result := nil;
end;

constructor TPointerList.Create;
begin
  FCurrent := nil;
  FFirst   := nil;
  FLast    := nil;
end;

destructor TPointerList.Destroy;
var
  PLI, Dummy: PPointerListItem;
begin
  PLI := FFirst;
  while Assigned(PLI) do begin
    Dummy := PLI^.Next;
    FreeMem(PLI);
    PLI := Dummy;
  end;
end;

procedure TPointerList.Add(p: Pointer);
var
  PLI: PPointerListItem;
begin
  PLI := GetMem(SizeOf(TPointerListItem));
  PLI^.Addr := p;
  PLI^.Next := nil;
  if Assigned(FLast) then
    PPointerListItem(FLast)^.Next := PLI;
  FLast := PLI;
  if not Assigned(FFirst) then
    FFirst := PLI;
end;

end.

