unit Progresses;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Dialogs, Forms, LResources, Controls, LCLIntf,
  ProgressForm;

type

  TStateEvent = procedure(Sender: TObject; const PerMille: Integer; const Progress, Detail: String; var aBreak: Boolean) of object;

  TProgressPart = record
    Caption: String;
    ShowSteps: QWord;
    RealSteps: QWord;
    Current: QWord;
    RoughTime: QWord;
    Time: QWord;
    OldCount: QWord;
  end;

  TProgressArray = array of TProgressPart;

  { TProgress }

  TProgress = class(TComponent)
  private
    FForm: TfrmProgress;
    FCaption: TCaption;
    FLogFileName: String;
    FLogFile: TextFile;
    FOnState: TStateEvent;
    FProgressParts: TProgressArray;
    FEmptyProgress: TProgressPart;
    FAbsRoughTime: QWord;
    FCurrentIndex: Integer;
    FRunning: Boolean;
    FVisible: Boolean;
    function GetManaged: Integer;
    function GetValueOfIndex(Index: Integer): TProgressPart;
    function IndexValid(Index: Integer): Boolean; inline;
    function IsStateCalling(Index: Integer): Boolean;
    procedure SetCaption(AValue: TCaption);
    procedure SetLogFileName(AValue: String);
    procedure SetVisible(AValue: Boolean);
  public
    procedure Clear;
    procedure Add(Caption: String; ShowSteps, RealSteps, RoughTime: QWord);
    procedure Start;
    procedure StartNext;
    procedure DefineRealSteps(Index: Integer; Steps: QWord); overload;
    procedure DefineRealSteps(Steps: QWord); overload;
    function IsStateCallingAddStep(Index: Integer): Boolean; overload;
    function IsStateCallingAddStep: Boolean; overload;
    function IsStateCallingSetStep(Index: Integer; Step: QWord): Boolean; overload;
    function IsStateCallingSetStep(Step: QWord): Boolean; overload;
    procedure State(const StateText: String);
    procedure Finish;
    procedure ShowProgressesRealTimes;
    constructor Create(AOwner: TComponent); override;
    property ValueOfIndex[Index: Integer]: TProgressPart read GetValueOfIndex; default;
    property Managed: Integer read GetManaged;              //0..1000 (per mille) -> Progressbar.Positions 0..1000
    property CurrentIndex: Integer read FCurrentIndex;
    property Running: Boolean read FRunning;
  published
    property Caption: TCaption read FCaption write SetCaption;
    property OnState: TStateEvent read FOnState write FOnState;
    property Visible: Boolean read FVisible write SetVisible;
    property LogFileName: String read FLogFileName write SetLogFileName;
  end;

var
  Progress: TProgress;

procedure register;

implementation

resourcestring

  SErrProgressUserBreak   = 'Stopped by User';
  CfrmProgressBtnBreak    = 'Abort';

procedure register;
begin
  RegisterComponents('System', [TProgress]);
end;

{ TProgress }

function TProgress.IndexValid(Index: Integer): Boolean;
begin
  Result := (Index >= 0) and (Index <= High(FProgressParts));
end;

function TProgress.IsStateCalling(Index: Integer): Boolean;
var
  aCount : QWord;
begin
  if FProgressParts[Index].ShowSteps = 0 then Exit(False);
  if FProgressParts[Index].RealSteps = 0 then Exit(False);
  if FProgressParts[Index].Current   = 0 then Exit(True);
  aCount := Round(FProgressParts[Index].Current * FProgressParts[Index].ShowSteps / FProgressParts[Index].RealSteps);
  Result := aCount > FProgressParts[Index].OldCount;
  if Result then
    FProgressParts[Index].OldCount := aCount;
end;

procedure TProgress.SetCaption(AValue: TCaption);
begin
  FCaption := AValue;
  if Assigned(FForm) then
    FForm.Caption := FCaption;
end;

procedure TProgress.SetLogFileName(AValue: String);
begin
  if FLogFileName = AValue then Exit;
  if FRunning then Exit;
  FLogFileName := AValue;
end;

procedure TProgress.SetVisible(AValue: Boolean);
begin
  if Running then Exit;
  FVisible := AValue;
end;

function TProgress.GetValueOfIndex(Index: Integer
  ): TProgressPart;
begin
  if not IndexValid(Index) then Exit(FEmptyProgress);
  Result := FProgressParts[Index];
end;

function TProgress.GetManaged: Integer;
var
  Cnt: DWord;
  i: Integer;
begin
  if FAbsRoughTime = 0 then Exit(0);
  Cnt := 0;
  for i := 0 to CurrentIndex - 1 do
    Cnt := Cnt + FProgressParts[i].RoughTime;
  if FProgressParts[CurrentIndex].RealSteps > 0 then
    Cnt := Cnt + Round(FProgressParts[CurrentIndex].Current * FProgressParts[CurrentIndex].RoughTime / FProgressParts[CurrentIndex].RealSteps);
  Result := Round(Cnt * 1000 / FAbsRoughTime);
end;

procedure TProgress.Clear;
begin
  SetLength(FProgressParts, 0);
  FAbsRoughTime := 0;
  FCurrentIndex := -1;
  FCaption      := '';
  FLogFileName  := '';
end;

procedure TProgress.Add(Caption: String; ShowSteps,
  RealSteps, RoughTime: QWord);
begin
  SetLength(FProgressParts, Length(FProgressParts) + 1);
  FProgressParts[High(FProgressParts)]           := FEmptyProgress;
  FProgressParts[High(FProgressParts)].Caption   := Caption;
  FProgressParts[High(FProgressParts)].ShowSteps := ShowSteps;
  FProgressParts[High(FProgressParts)].RealSteps := RealSteps;
  FProgressParts[High(FProgressParts)].RoughTime := RoughTime;
  FAbsRoughTime := FAbsRoughTime + RoughTime;
end;

procedure TProgress.Start;
begin
  FRunning := True;
  if Length(FLogFileName) > 0 then begin
    AssignFile(FLogFile, FLogFileName);
    Rewrite(FLogFile);
  end;
  if Visible then begin
    FForm                  := TfrmProgress.Create(nil);
    FForm.Caption          := FCaption;
    FForm.btnBreak.Down    := False;
    FForm.btnBreak.Caption := CfrmProgressBtnBreak;
    OnState                := @FForm.ShowState;
    FForm.Show;
    State('');
  end;
  FCurrentIndex := 0;
  State('');
  if Length(FLogFileName) > 0 then WriteLn(FLogFile, FProgressParts[0].Caption);
  FProgressParts[0].Time := GetTickCount64;
end;

procedure TProgress.StartNext;
begin
  if not IndexValid(CurrentIndex + 1) then Exit;
  FCurrentIndex := CurrentIndex + 1;
  if FCurrentIndex = 0 then begin
    Start;
    Exit;
  end;
  State('');
  if Length(FLogFileName) > 0 then WriteLn(FLogFile, FProgressParts[FCurrentIndex].Caption);
  FProgressParts[FCurrentIndex].Time := GetTickCount64;
  FProgressParts[FCurrentIndex - 1].Current := FProgressParts[FCurrentIndex - 1].ShowSteps;
end;

procedure TProgress.DefineRealSteps(Index: Integer;
  Steps: QWord);
begin
  if not IndexValid(Index) then Exit;
  FProgressParts[Index].RealSteps := Steps;
  FProgressParts[Index].Current   := 0;
end;

procedure TProgress.DefineRealSteps(Steps: QWord);
begin
  DefineRealSteps(FCurrentIndex, Steps);
end;

function TProgress.IsStateCallingAddStep(Index: Integer
  ): Boolean;
begin
  if not IndexValid(Index) then Exit(False);
  Inc(FProgressParts[Index].Current);
  Result := IsStateCalling(Index);
end;

function TProgress.IsStateCallingAddStep: Boolean;
begin
  Result := IsStateCallingAddStep(FCurrentIndex);
end;

function TProgress.IsStateCallingSetStep(Index: Integer;
  Step: QWord): Boolean;
begin
  if not IndexValid(Index) then Exit(False);
  FProgressParts[Index].Current := Step;
  Result := IsStateCalling(Index);
end;

function TProgress.IsStateCallingSetStep(Step: QWord
  ): Boolean;
begin
  Result := IsStateCallingSetStep(FCurrentIndex, Step);
end;

procedure TProgress.State(const StateText: String);
var
  aBreak: Boolean;
begin
  if Length(FLogFileName) > 0 then WriteLn(FLogFile, '  ', StateText);
  if not Assigned(FOnState) then Exit;
  aBreak := False;
  FOnState(Self, Progress.Managed, Progress[Progress.CurrentIndex].Caption, StateText, aBreak);
  if aBreak then begin
    Finish;
    FRunning := False;
    raise Exception.Create(SErrProgressUserBreak);
  end;
end;

procedure TProgress.Finish;
var
  i: Integer;
begin
  if Visible and Assigned(FForm) then FreeAndNil(FForm);
  if not Running then Exit;
  if Length(FLogFileName) > 0 then CloseFile(FLogFile);
  if High(FProgressParts) = 0 then Exit;
  for i := 0 to High(FProgressParts) - 1 do
    FProgressParts[i].Time := FProgressParts[i + 1].Time - FProgressParts[i].Time;
  FProgressParts[High(FProgressParts)].Time := GetTickCount64 - FProgressParts[High(FProgressParts)].Time;
  FRunning := False;
end;

procedure TProgress.ShowProgressesRealTimes;
var
  s: String;
  i: Integer;
begin
  //  for i := 0 to High(FProgressParts) do writeln(FProgressParts[i].Time);  //Auskommentieren, um genaue Zeiten zu ermitteln
  s := '';
  for i := 0 to High(FProgressParts) do
    s := s + '[' + IntToStr(FProgressParts[i].Time) + ']ms: ' + FProgressParts[i].Caption + LineEnding;
  ShowMessage(s);
end;

constructor TProgress.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not Assigned(Progress) then Progress := Self;
  FEmptyProgress.Caption   := '';
  FEmptyProgress.ShowSteps := 0;
  FEmptyProgress.RealSteps := 0;
  FEmptyProgress.Current   := 0;
  FEmptyProgress.RoughTime := 0;
  FEmptyProgress.OldCount  := 0;
  Clear;
  FVisible := False;
  FRunning := False;
  FForm    := nil;
end;

initialization

  {$I progress_package.lrs}

end.

